% Read DAT5 Compression Data
%
clear all;
close all;

% ---------------------------
% Initial Parameters
%
File.name   = 'SampleDAT.DAT';
FORMAT = {'ubit24','','ubit20','','ubit16','','ubit12','ubit8'};
SUBST  = [ 2^23   , 0, 2^19   , 0, 2^15   , 0, 2^11   , 2^7]; 
YY00   = [1900 2000];
% ---------------------------
% Open File
%
[fid, message] = fopen(File.name,'r','b');
if fid == -1, error(message); end

% ---------------------------
% Read Data
%
blk = 1;
while 1
    disp(['Block Number : ',num2str(blk)]);
    if blk == 1;
        fread(fid,  8,'ubit4');                         % (  4Bytes) 0x242424
    else
        while 1
            if feof(fid)
                break;
            elseif fread(fid,1,'ubit4') == 2
                break;
            end
        end
        if feof(fid), break, end
        fread(fid,  7,'ubit4');
    end
% ---------------------------
% Read Block Header
%
    File.SN          = fread(fid,  1,'ubit16');     % (  2Bytes) Serial Number
    tmp              = fread(fid, 16,'ubit4')';     % (  8Bytes) BCD
    tmp              = (reshape(tmp,2,8)'*[10;1])';
    File.StTm(blk)   = datenum([YY00((tmp(1)<50)+1)+tmp(1),tmp(2:5),tmp(6)+tmp(7:8)*[0.01;0.0001]]);
    File.SmpFrq      = round(10^6/fread(fid,  1,'ubit32')); % (  4Bytes) Sampling Rate (micro sec)
    File.chn         = fread(fid,  1,'ubit16');  	% (  2Bytes) Number of Channels
    File.NumSmp(blk) = fread(fid,  1,'ubit32');     % (  4Bytes) Number of Sampling Data
    File.model       = fread(fid,  1,'ubit16');     % (  2Bytes) Model Number
    if blk > 1 && (File.SN ~= Prev.SN || File.SmpFrq ~= Prev.SmpFrq || File.chn ~= Prev.chn || File.model ~= Prev.model)
            error('Header informatin is inconsistent with the previous Header.');
    end
    Prev = File;
    tmp              = char(fread(fid,358,'ubit8'))'; % (358Bytes) GPS status
    tmp(tmp==char(0))= ' ';
    tmp              = textscan(tmp,'%s');
    tmp              = tmp{1};
    Idx = find(strcmp(tmp,'GPS'));
    if ~isempty(Idx)
        if exist('GPS','var')
            k = length(GPS.valid);
        else
            k = 0;
        end
        for ii = 1:length(Idx)
            if strcmp(tmp{Idx(ii)+1},'VALID')
                GPS.valid(k+ii)   = 1;
                GPS.lat{k+ii}     = tmp{Idx(ii)+2};
                GPS.lon{k+ii}     = tmp{Idx(ii)+3};
                tmp2              = sscanf(tmp{Idx(ii)+4},'%2d')';
                GPS.GPStime(k+ii) = datenum([YY00((tmp2(1)<50)+1)+tmp2(1),tmp2(2:6)]);
                tmp2              = sscanf(tmp{Idx(ii)+5},'%2d')';
                GPS.RECtime(k+ii) = datenum([YY00((tmp2(1)<50)+1)+tmp2(1),tmp2(2:5),tmp2(6)+str2double(tmp{Idx(ii)+6})/1000000]);
            else
                GPS.valid(k+ii)   = 0;
                GPS.lat{k+ii}     = '';
                GPS.lon{k+ii}     = '';
                GPS.GPStime(k+ii) = 0;
                GPS.RECtime(k+ii) = 0;
            end
        end
    end
% ---------------------------
% Parameter Initialize
%
    if blk == 1
        OFFSET = 0;
        DATA = zeros(File.SmpFrq,File.chn);
    else
        OFFSET = size(DATA,1);
        DATA(OFFSET+1:OFFSET+File.NumSmp(blk),:) = 0;
    end
% ---------------------------
% Read Block Data
%
    NumSmp = 0;
    while 1
        if NumSmp >= File.NumSmp(blk), break; end
        for ii = 1:File.chn
            ChHEAD = fread(fid,8,'ubit1')';
%             disp([num2str(NumSmp),' : ',num2str(ChHEAD)]);
            k = [ChHEAD(1:2),ChHEAD(4)]*[4;2;1]+1;  % 1:24bit, 3:20bit, 5:16bit, 7:12bit, 8:8bit
            if ChHEAD(3) == 0
                DATA(OFFSET+NumSmp+1:OFFSET+NumSmp+File.SmpFrq,ii) = cumsum([fread(fid,1,'ubit24')-2^23;(fread(fid,File.SmpFrq-1,FORMAT{k})-SUBST(k))*-1]); % Cumulative Sum
            elseif ChHEAD(3) == 1
                DATA(OFFSET+NumSmp+1:OFFSET+NumSmp+2,ii) = fread(fid,2,'ubit24')-2^23;
                for jj = OFFSET+NumSmp+3:OFFSET+NumSmp+File.SmpFrq
                    DATA(jj,ii) = (fread(fid,1,FORMAT{k})-SUBST(k))-DATA(jj-2,ii)+DATA(jj-1,ii)*2;
                end
            else
                error(['Compression Type is wrong : ',num2str(ChHEAD(3))]);
            end
            if k == 3 && ChHEAD(3) == 0   	% 20bit
                fread(fid,1,'ubit20');
            elseif k == 7 && ChHEAD(3) == 0	% 12bit
             	fread(fid,1,'ubit12');
            end
        end
        NumSmp = NumSmp+File.SmpFrq;
    end
    blk = blk+1;
end
fclose(fid);
DATA = DATA*4/2^23;

figure(1);
TIME = (0:size(DATA,1)-1)'/File.SmpFrq;
subplot(3,1,1), plot(TIME,DATA(:,1)); grid on; legend('ch1')
title(datestr(File.StTm(1),'yyyy/mm/dd HH:MM:SS'));
xlabel('Time/Sec'); ylabel('Amplitude/V');
subplot(3,1,2), plot(TIME,DATA(:,2)); grid on; legend('ch2')
xlabel('Time/Sec'); ylabel('Amplitude/V');
subplot(3,1,3), plot(TIME,DATA(:,3)); grid on; legend('ch3')
xlabel('Time/Sec'); ylabel('Amplitude/V');

clear ChHEAD FORMAT Idx NumSmp OFFSET Prev SUBST YY00 blk fid ii jj k tmp tmp2 ans message
