% Main Program for Noise Analisis
%
% *** UPDATES ***
% 2010/Dec/03(Fri) : Add ReadDAT.m
% 2008/Jul/18(Fri) : Original by Koji MIYAKAWA
%
clear all;
% -------------------------------------------------------------------------
% Parameter Initialization (Essential)
%

% FileNm  = 'SampleWIN.win';      % For WIN data
% ChTBLfn = 'SampleWIN.ch';       % For WIN data

FileNm  = 'SampleDAT.DAT';      % For DAT data
ChTBLfn = 'SampleDAT.ch';       % For DAT data
% ChTBLfn = '';                   % For DAT data

% -------------------------------------------------------------------------
% Parameter Initialization (Supplemental)
% Basically, empty cell "[]" means that its default parameter is used.
%
% Set Default Axes Position
set(0,'DefaultAxesPosition',[0.08 0.05 0.88 0.91]); % Default is [0.13 0.11 0.775 0.815]
set(0,'defaultFigureColor',[1 1 1]);                % Default is [0.8000 0.8000 0.8000]

% Read and write a WIN file (ReadWIN.m)
LoadMat  = 'No';   % Load "*.mat" file instead of WIN file if exists ('Yes' or 'No'). Default is 'Yes'.
MatSave  = 'No';   % Save loaded WIN data as "*.mat" file after ReadWIN command ('Yes' or 'No'). Default is 'Yes'.
ChIDcell = {};      % Channel ID(s) to be extracted from the WIN file (cell array). Ex) {'00A0','00B1'} or {} (All the channels data are extracted)
StEdTime = 600;      % Start and end time to be extracted. Time format is [yyyy mm dd HH MM SS]. Default is whole time period. Ex) {[StartTime] [EndTime]} (cell array), or {[StartTime] DataLength(sec)} or DataLength(sec) or []

% Filtering and decimation (Filter.m)
BPFfc    = [];      % Band Pass Filter Parameter [LowCut(Hz) HighCut(Hz)] Ex) [10 20] or [NaN 20] (LPF) or [10 NaN] (HPF) or [] (No filter is applied)
DECI     = [];      % Decimation Frequency. This must be interger fractions fo original sampling frequency. Default is none.

% PSD and spectrogram calculation, and differential/integral (PSD_SPGRM.m)
PSD_WndwSec   = 10; % Window length in second for the PSD calculation. Default is the length where data is divided into eight time windows.
PSD_OvlpPct   = 0;  % Percentage of overlaps of windowed sections for the PSD calculation. Default is 50%.
SPGRM_WndwSec = 10; % Window length in second for the spectrogram calculation. Default is the length where data is divided into eight time windows.
SPGRM_OvlpPct = 0;  % Percentage of overlaps of windowed sections for the spectrogram calculation. Default is 50%.
AppResp       = 'No'; % Apply sensor response based on the ChTbl.Damp and ChTbl.NatFrq. 'Yes' or 'No' or []. Default is 'No'.
AppRespChs    = {'0200'}; % Target channel ID(s) that ApplyResp is applied (cell array). Ex) {'00A0','00B1'} or {} (All the channels are applied)
DiffIntg      = 'none'; % 'Differential' or 'Integral' or 'none' or '' or [] (Empty is the same as 'none')
DiffIntgChs   = {'0200'}; % Target channel ID(s) that DiffIntgrl is applied (cell array). Ex) {'00A0','00B1'} or {} (All the channels are applied)

% Plot waveform, spectrogram and PSD in one figure (Plot_SPGRM.m)
ChID      = {}; % Channel ID(s) to be plotted (char). Ex) {'00A0','00A1'} or {'00A0'} or {} (Empty means the lowest ChID is selected) 
Xunit     = ''; % Unit of Xaxis. 'Sec' or 'Minute' or 'Hour' or 'Clock'. Default is 'Sec'
DataXLIM  = []; % Xaxis limits (time) of the waveform plot. [Xmin Xmax] or [] (Display whole range). If Xunit is 'Clock', DataXLIM must be date datevec format like {[yyyy mm dd HH MM SS],[yyyy mm dd HH MM SS]}.
DataYLIM  = []; % Yaxis limits (time) of the waveform plot. [Ymin Ymax] or [] (Display whole range)
FreqLIM   = []; % Xaxis limits (frequency) of the spectrogram and PSD plot. [Fmin Fmax] or [] (Display whole range).
CLIM      = []; % Color limits of the  spectrogram. [Cmin Cmax] or [] (Display whole range)
PsdYLIM   = []; % Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
PsdXScale = ''; % Xaxis scale of the  PSD plot. 'log' or 'linear' or '' or []. Default is 'linear'.
UsgsNM    = ''; % Plot USGS noise model(NHNM/NLNM) on the PSD axes. 'Yes' or 'No' or ''. Default is 'Yes'
PlotSkip  = []; % Number of skipping data points for waveform plot. Default is 1 (no skip).
MvAvg     = []; % Moving average for smoothing the PSD plot. Default is 1 (do not apply moving average).

% Plot waveform like a drum recording (Plot_Drum.m)
ChIDdrum     = {};  % Channel ID(s) to be plotted (char). Ex) {'00A0','00A1'} or {'00A0'} or {} (Empty means the lowest ChID is selected) 
UnitSc       = [];  % Unit scale of Yaxis. Default is (max(DATA)-min(DATA))/5.
% UnitSc       = [0.0007];  % Unit scale of Yaxis. Default is (max(DATA)-min(DATA))/5.
TimeSc       = 'Hour';  % Time Scale. 'Day' or 'Hour' only. Default is 'Day'.
TarDateTime  = [];  % Target Date & Hour. Ex) [2008 07 18] for 'Day' plot, and [2008 07 18 12] for 'Hour' plot. Default is the date & hour of the first data point.
PlotSkipDrum = [];  % Number of skipping data points for waveform plot. Default is 1 (no skip).

% Plot multiple waveforms (Plot_WVs.m)
XunitWvs    = ''; % Unit of Xaxis. 'Sec' or 'Minute' or 'Hour' or 'Clock'. Default is 'Sec'.
XLIMwvs     = []; % Xaxis limits (time). [Xmin Xmax] or [] (Display whole range). If XunitWvs is 'Clock', XLIMwvs must be date datevec format like {[yyyy mm dd HH MM SS],[yyyy mm dd HH MM SS]}.
YRange      = []; % Yaxis range (One-sided amplitude from the YMEAN). Default is whole range.
% YRange      = [16e-4]; % Yaxis range (One-sided amplitude from the YMEAN). Default is whole range.
PlotSkipWvs = []; % Number of skipping data points for waveform plot. Default is 1 (no skip).

% Plot multiple PSDs in multiple axeses (Plot_PSDs.m)
XScalePsds  = ''; % Xaxis Scale of the PSD plot. 'log' or 'linear' or '' or []. Default is 'linear'.
XLIMpsds    = []; % Xaxis limits (frequency) of the PSD plot. [Fmin Fmax] or [] (Display whole range).
% XLIMpsds    = [1e-1 1e2]; % Xaxis limits (frequency) of the PSD plot. [Fmin Fmax] or [] (Display whole range).
YLIMpsds    = []; % Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
% YLIMpsds    = [1e-15 1e-8]; % Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
UsgsNMpsds  = ''; % Plot USGS noise model(NHNM/NLNM) on the PSD axeses. 'Yes' or 'No' or ''. Default is 'Yes'
MvAvgPsds   = []; % Moving average for smoothing the PSD plot. Default is 1 (do not apply moving average).

% Plot multiple PSDs in one axes (Plot_PSD.m)
XScalePsd   = 'log'; % Xaxis Scale of the PSD plot. 'log' or 'linear' or '' or []. Default is 'linear'.
XLIMpsd     = []; % Xaxis limits (frequency) of the PSD plot. [Fmin Fmax] or [] (Display whole range).
% XLIMpsd     = [1e-1 1e2]; % Xaxis limits (frequency) of the PSD plot. [Fmin Fmax] or [] (Display whole range).
YLIMpsd     = []; % Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
% YLIMpsd     = [1e-15 1e-8]; % Yaxis limits of the PSD plot. [Ymin Ymax] or [] (Display whole range)
UsgsNMpsd   = ''; % Plot USGS noise model(NHNM/NLNM) on the PSD axes. 'Yes' or 'No' or ''. Default is 'Yes'
MvAvgPsd    = []; % Moving average for smoothing the PSD plot. Default is 1 (do not apply moving average).

% =========================================================================
% Read & Write, and PSD & Spectrogram Calculation
%
if strcmpi(LoadMat,'Yes') && exist([FileNm,'.mat'],'file')
    fprintf('Load from MAT file : %s\n\n',[FileNm,'.mat']);
    load([FileNm,'.mat']);   % Load MAT file
else
    fprintf('Load from Waveform file : %s\n\n',FileNm);
    [~,~,Ext] = fileparts(FileNm);
    if strcmpi(Ext,'.dat')
        [DATA,File,ChTbl] = ReadDAT(FileNm,ChTBLfn,ChIDcell,StEdTime);   % Extract all the channels data of the WIN file into Matlab variables
    else
        [DATA,File,ChTbl] = ReadWIN(FileNm,ChTBLfn,ChIDcell,StEdTime);   % Extract all the channels data of the WIN file into Matlab variables
    end    
    [DATA,File] = Filter(DATA,File,BPFfc,DECI);               % Band pass filtering and decimation
    [DATA,File,ChTbl,PSD,SPGRM] = PSD_SPGRM(DATA,File,ChTbl,PSD_WndwSec,PSD_OvlpPct,SPGRM_WndwSec,SPGRM_OvlpPct,AppResp,AppRespChs,DiffIntg,DiffIntgChs);
    if strcmpi(MatSave,'Yes')   % Save to MAT file
        fprintf('Save to MAT file   : %s\n\n',[FileNm,'.mat']);
        save([FileNm,'.mat'],'DATA','File','ChTbl','PSD','SPGRM');
    end
end

% =========================================================================
% Display Extracted Data Information
%
fprintf('***** Extracted WIN data information *****\n');
fprintf('Start Time    : %s %s\n',File.StDate,File.StTime);
fprintf('End   Time    : %s%s\n',datestr(File.StNum+File.RecSec/86400,'yyyy/mm/dd'),datestr(File.StNum+File.RecSec/86400,'(ddd) HH:MM:SS'));
fprintf('Record Length : %G(sec)\n',File.RecSec);
fprintf('ChID(s)       :');
for ii = 1:length(ChTbl.ChID)
    fprintf(' %s(%GHz)',ChTbl.ChID{ii},File.SmpFrq(File.ColChs(ii)));
end
fprintf('\n******************************************\n\n');

% =========================================================================
% Figure Plot
%

% ---------------------------
% Plot_WV_SPGRM_PSD
%
if isempty(ChID), ChID = {''}; end
for ii = 1:length(ChID)
    figure(ii); clf;
    Plot_SPGRM(DATA,File,ChTbl,PSD,SPGRM,ChID{ii},Xunit,DataXLIM,DataYLIM,FreqLIM,CLIM,PsdYLIM,PsdXScale,UsgsNM,PlotSkip,MvAvg);
end

% ---------------------------
% Plot_Drum
%
if isempty(ChIDdrum), ChIDdrum = {''}; end
for ii = 1:length(ChIDdrum)
    figure(ii+9); clf;
    Plot_Drum(DATA,File,ChTbl,ChIDdrum{ii},UnitSc,TimeSc,TarDateTime,PlotSkipDrum);
end

% ---------------------------
% Plot_WVs, Plot_PSDs, and Plot_PSD
%
if length(ChTbl.ChID) > 1
    figure(20); clf;        % Plot_WVs
    Plot_WVs(DATA,File,ChTbl,XunitWvs,XLIMwvs,YRange,PlotSkipWvs);
    figure(30); clf;        % Plot_PSDs
    Plot_PSDs(File,ChTbl,PSD,XScalePsds,XLIMpsds,YLIMpsds,UsgsNMpsds,MvAvgPsds);
    figure(40); clf;        % Plot_PSD
    Plot_PSD(File,ChTbl,PSD,XScalePsd,XLIMpsd,YLIMpsd,UsgsNMpsd,MvAvgPsd);
end
